/*******************************************************************************
 * Copyright (C) 2024 Juan Neyra
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
#pragma once

#ifndef SC_INCLUDE_DYNAMIC_PROCESSES
#define SC_INCLUDE_DYNAMIC_PROCESSES
#endif

#include "NetworkInterfaceTlm.h"
#include "utils/memory_manager.h"

class NetworkInterfaceTlmCS : public NetworkInterfaceTlm {
public:
    SC_HAS_PROCESS(NetworkInterfaceTlmCS);

    NetworkInterfaceTlmCS(sc_module_name nm, Node& node, uint8_t max_pos[3]);

    ~NetworkInterfaceTlmCS() override;

    // To Circuit Switching NoC
    ni_init_socket initiator_cs;
    ni_targ_socket target_cs;

private:
    /** 
     * Sends flit to packet switching noc or circuit switching noc
     *
     * @param p   parent packet of the flit to send
     * @param f   flit to send
     */
    void send_flit(Packet* p, Flit* f) override;

    /** 
     * Logs the arrival of a flit. After processing, sends a 
     * response if needed
     *
     * @param trans   TLM transaction object
     */
    void receive_and_process_flit(tlm_gp& trans) override;

    /**
     * Send end request
     *
     * @param trans TLM generic payload object
     */
    tlm::tlm_sync_enum send_end_req(tlm_gp& trans) override;
};
